@@ -33,7 +33,7 @@ int check_bin(char *tbinary)
     } else if (child_pid < 0) {	/* fork failed */
 	int err = errno;
 
-	if (errno == EAGAIN) {	/* process table full */
+ 	if (errno == EAGAIN) {	/* process table full */
 #if USE_SYSLOG
 	    syslog(LOG_ERR, "process table is full!");
 #endif				/* USE_SYSLOG */
@@ -53,32 +53,40 @@ int check_bin(char *tbinary)
 	do {
 	    ret = waitpid(-1, &result, WNOHANG);
 	    err = errno;
-	    /* check if actual child terminated */
-	    if (ret == child_pid)
-		child_pid = 0;
-	} while (ret > 0 && WEXITSTATUS(result) == 0);
+	} while (ret > 0 && WIFEXITED(result) != 0 && WEXITSTATUS(result) == 0);
 
-	/* check result */
-	/* if one of the scripts returns an error code just return that code */
-	res = WEXITSTATUS(result);
-	if (res != 0) {
+	/* check result: */
+	/* ret < 0 			=> error */
+	/* ret == 0			=> no more child returned, however we may already have caught the actual child */
+	/* WIFEXITED(result) == 0	=> child did not exit normally but was killed by signal which was not caught */
+	/* WEXITSTATUS(result) != 0	=> child returned an error code */
+	if (ret > 0) {
+		if (WIFEXITED(result) != 0) {
+			/* if one of the scripts returns an error code just return that code */
 #if USE_SYSLOG
-	    syslog(LOG_ERR, "test binary returned %d", res);
+			syslog(LOG_ERR, "test binary returned %d", WEXITSTATUS(result));
 #endif				/* USE_SYSLOG */
-	    return (res);
-	}
-	/* give an error in case there are still old childs running */
-	/* in fact if an old one hangs around we already got an error */
-	/* message in earlier rounds */
-	if (ret == 0 && child_pid != 0) {
+		    	return (WEXITSTATUS(result));
+		} else if (WIFSIGNALED(result) != 0)  {
+			/* if one of the scripts was killed return ECHKILL */
 #if USE_SYSLOG
-	    errno = err;
-	    syslog(LOG_ERR, "child %d did not exit immediately (error = %d = '%m')", child_pid, err);
+			syslog(LOG_ERR, "test binary was killed by uncaught signal %d", WTERMSIG(result));
+#endif				/* USE_SYSLOG */
+		    	return (ECHKILL);
+		}
+	} else {
+		/* in case there are still old childs running due to an error */
+		/* log that error */
+		if (err != 0 && err != ECHILD) {
+#if USE_SYSLOG
+		    errno = err;
+		    syslog(LOG_ERR, "child %d did not exit immediately (error = %d = '%m')", child_pid, err);
 #else				/* USE_SYSLOG */
-	    perror(progname);
+		    perror(progname);
 #endif				/* USE_SYSLOG */
-	    if (softboot)
-		return (err);
+		    if (softboot)
+			return (err);
+		}
 	}
     }
     return (ENOERR);
