@@ -20,6 +20,7 @@
 #include <arpa/inet.h>
 #include <sys/mman.h>
 #include <sys/wait.h>
+#define __USE_GNU
 #include <string.h>
 
 #if !defined(__GLIBC__)		/* __GLIBC__ */
@@ -45,11 +46,14 @@ int verbose = FALSE;
 #define CHANGE		"change"
 #define DEVICE		"watchdog-device"
 #define	FILENAME	"file"
+#define INTERFACE	"interface"
 #define INTERVAL	"interval"
 #define MAXLOAD1	"max-load-1"
 #define MAXLOAD5	"max-load-5"
 #define MAXLOAD15	"max-load-15"
 #define MAXTEMP		"max-temperature"
+#define MINMEM		"min-memory"
+#define SERVERPIDFILE	"pidfile"
 #define PING		"ping"
 #define PRIORITY	"priority"
 #define REALTIME	"realtime"
@@ -58,9 +62,9 @@ int verbose = FALSE;
 #define TESTBIN		"test-binary"
 
 pid_t pid;
-int softboot = FALSE, watchdog = -1, load = -1, temp = -1, tint = 10, schedprio = 1;
+int softboot = FALSE, watchdog = -1, load = -1, mem = -1, temp = -1, tint = 10, schedprio = 1;
 char *tempname = NULL, *devname = NULL, *admin = "root", *progname;
-int maxload1 = 12, maxload5 = 9, maxload15 = 6;
+int maxload1 = 0, maxload5 = 0, maxload15 = 0, minpages = 0;
 int maxtemp = 120;
 
 #if defined(_POSIX_MEMLOCK)
@@ -93,7 +97,7 @@ static int repair(char *rbinary, int result)
 {
     pid_t child_pid;
     char parm[5];
-    char ret;
+    int ret;
 
     /* no binary given, we have to reboot */
     if (rbinary == NULL)
@@ -173,7 +177,7 @@ static void do_check(int res, char *rbinary)
     wd_action(keep_alive(), rbinary);
 }
 
-struct list *file = NULL, *target = NULL;
+struct list *file = NULL, *target = NULL, *pidfile = NULL, *iface = NULL;
 char *tbinary, *rbinary, *admin;
 
 static void add_list(struct list **list, char *name)
@@ -216,6 +220,7 @@ static void read_config(char *filename, char *progname)
 	perror(progname);
 	exit(1);
     }
+
     while (!feof(wc)) {
 	char line[CONFIG_LINE_LEN];
 
@@ -247,7 +252,6 @@ static void read_config(char *filename, char *progname)
 
 	    /* now check for an option */
 	    if (strncmp(line + i, FILENAME, strlen(FILENAME)) == 0) {
-	    	
 		if (spool(line, &i, strlen(FILENAME)))
 		    fprintf(stderr, "Ignoring invalid line in config file:\n%s\n", line);
 		else
@@ -267,11 +271,21 @@ static void read_config(char *filename, char *progname)
 		    fprintf(stderr, "Duplicate change interval option in config file. Ignoring first entry.\n");
 
 		file->parameter.file.mtime = atoi(line + i);
+	    } else if (strncmp(line + i, SERVERPIDFILE, strlen(SERVERPIDFILE)) == 0) {
+		if (spool(line, &i, strlen(SERVERPIDFILE)))
+		    fprintf(stderr, "Ignoring invalid line in config file:\n%s\n", line);
+		else
+		    add_list(&pidfile, strdup(line + i));
 	    } else if (strncmp(line + i, PING, strlen(PING)) == 0) {
 		if (spool(line, &i, strlen(PING)))
 		    fprintf(stderr, "Ignoring invalid line in config file:\n%s\n", line);
 		else
 		    add_list(&target, strdup(line + i));
+	    } else if (strncmp(line + i, INTERFACE, strlen(INTERFACE)) == 0) {
+		if (spool(line, &i, strlen(INTERFACE)))
+		    fprintf(stderr, "Ignoring invalid line in config file:\n%s\n", line);
+		else
+		    add_list(&iface, strdup(line + i));
 	    } else if (strncmp(line + i, REALTIME, strlen(REALTIME)) == 0) {
 		(void)spool(line, &i, strlen(REALTIME));
 		realtime = (strncmp(line + i, "yes", 3) == 0) ? TRUE : FALSE;
@@ -339,6 +353,12 @@ static void read_config(char *filename, char *progname)
 			maxload5 = atol(line + i);
 			gotload5 = TRUE;
 		}
+	    } else if (strncmp(line + i, MINMEM, strlen(MINMEM)) == 0) {
+		if (spool(line, &i, strlen(MINMEM)))
+			fprintf(stderr, "Ignoring invalid line in config file:\n%s\n", line);
+		else {
+			minpages = atol(line + i);
+		}
 	    } else {
 		fprintf(stderr, "Ignoring invalid line in config file:\n%s\n", line);
 	    }
@@ -375,7 +395,8 @@ int main(int argc, char *const argv[])
 	{"sync", no_argument, NULL, 's'},
 	{"no-action", no_argument, NULL, 'q'},
 	{"verbose", no_argument, NULL, 'v'},
-	{"softboot", no_argument, NULL, 'b'}
+	{"softboot", no_argument, NULL, 'b'},
+	{NULL, 0, NULL, 0}
     };
     long count = 0L;
 #else				/* USE_SYSLOG */
@@ -386,7 +407,8 @@ int main(int argc, char *const argv[])
 	{"force", no_argument, NULL, 'f'},
 	{"sync", no_argument, NULL, 's'},
 	{"no-action", no_argument, NULL, 'q'},
-	{"softboot", no_argument, NULL, 'b'}
+	{"softboot", no_argument, NULL, 'b'},
+	{NULL, 0, NULL, 0}
     };
 #endif				/* USE_SYSLOG */
 
@@ -446,14 +468,15 @@ int main(int argc, char *const argv[])
 	fprintf(stderr, "To force this interval length use the -f option.\n");
 	exit(1);
     }
-    if (maxload1 < MINLOAD && !force) {
+    
+    if (maxload1 > 0 && maxload1 < MINLOAD && !force) {
 	fprintf(stderr, "%s error:\n", progname);
 	fprintf(stderr, "Using this maximal load average might reboot the system to often!\n");
 	fprintf(stderr, "To force this load average use the -f option.\n");
 	exit(1);
     }
     
-    /* set up pinging if in network mode */
+    /* set up pinging if in ping mode */
     if (target != NULL) {
 	for (act = target; act != NULL; act = act->next) {
 	    struct protoent *proto;
@@ -540,12 +563,12 @@ int main(int argc, char *const argv[])
     openlog(progname, LOG_PID, LOG_DAEMON);
     sprintf(log, "starting daemon (%d.%d):", MAJOR_VERSION, MINOR_VERSION);
     
-    sprintf(log + strlen(log), " int=%ds realtime=%s sync=%s soft=%s mla=%d ping=",
+    sprintf(log + strlen(log), " int=%ds realtime=%s sync=%s soft=%s mla=%d mem=%ld ping=",
 	    tint,
 	    realtime ? "yes" : "no",
 	    sync_it ? "yes" : "no",
 	    softboot ? "yes" : "no",
-	    maxload1);
+	    maxload1, minpages);
 	    
     if (target == NULL)
             sprintf(log + strlen(log), "none ");
@@ -560,6 +583,20 @@ int main(int argc, char *const argv[])
             for (act = file; act != NULL; act = act->next)
                 sprintf(log + strlen(log), "%s:%d%c", act->name, act->parameter.file.mtime, (act->next != NULL) ? ',' : ' ');
 
+    sprintf(log + strlen(log), "pidfile=");
+    if (pidfile == NULL)
+            sprintf(log + strlen(log), "none ");
+    else
+            for (act = pidfile; act != NULL; act = act->next)
+                sprintf(log + strlen(log), "%s%c", act->name, (act->next != NULL) ? ',' : ' ');                
+
+    sprintf(log + strlen(log), "iface=");
+    if (iface == NULL)
+            sprintf(log + strlen(log), "none ");
+    else
+            for (act = iface; act != NULL; act = act->next)
+                sprintf(log + strlen(log), "%s%c", act->name, (act->next != NULL) ? ',' : ' ');                
+
     sprintf(log + strlen(log), "test=%s repair=%s alive=%s temp=%s to=%s no_act=%s",
 	    (tbinary == NULL) ? "none" : tbinary,
 	    (rbinary == NULL) ? "none" : rbinary,
@@ -584,15 +621,31 @@ int main(int argc, char *const argv[])
 	    /* we can use watchdog even if there is no watchdog device */
 	}
     }
-    /* open the load average file */
-    load = open("/proc/loadavg", O_RDONLY);
-    if (load == -1) {
+
+    if (maxload1 > 0) {
+	/* open the load average file */
+	load = open("/proc/loadavg", O_RDONLY);
+	if (load == -1) {
 #if USE_SYSLOG
-	syslog(LOG_ERR, "cannot open /proc/loadavg (errno = %d = '%m')", errno);
+		syslog(LOG_ERR, "cannot open /proc/loadavg (errno = %d = '%m')", errno);
 #else				/* USE_SYSLOG */
-	perror(progname);
+		perror(progname);
 #endif				/* USE_SYSLOG */
+	}
     }
+
+    if (minpages > 0) {
+	/* open the memory info file */
+	mem = open("/proc/meminfo", O_RDONLY);
+	if (mem == -1) {
+#if USE_SYSLOG
+		syslog(LOG_ERR, "cannot open /proc/meminfo (errno = %d = '%m')", errno);
+#else				/* USE_SYSLOG */
+		perror(progname);
+#endif				/* USE_SYSLOG */
+	}
+    }
+    
     if (tempname != NULL && no_act == FALSE) {
 	/* open the temperature file */
 	temp = open(tempname, O_RDONLY);
@@ -652,6 +705,9 @@ int main(int argc, char *const argv[])
 
 	/* check load average */
 	do_check(check_load(), rbinary);
+	
+	/* check free memory */
+	do_check(check_memory(), rbinary);
 
 	/* check temperature */
 	do_check(check_temp(), rbinary);
@@ -659,8 +715,16 @@ int main(int argc, char *const argv[])
 	/* in filemode stat file */
 	for (act = file; act != NULL; act = act->next)
 	    do_check(check_file_stat(act), rbinary);
+	    
+	/* in pidmode kill -0 processes */
+	for (act = pidfile; act != NULL; act = act->next)
+	    do_check(check_pidfile(act), rbinary);
 
-	/* in network mode ping the ip address */
+	/* in network mode check the given devices for input */
+	for (act = iface; act != NULL; act = act->next)
+	    do_check(check_iface(act), rbinary);
+	    
+	/* in ping mode ping the ip address */
 	for (act = target; act != NULL; act = act->next)
 	    do_check(check_net(act->name, act->parameter.net.sock_fp, act->parameter.net.to, act->parameter.net.packet, tint / 3), rbinary);
 
